[![WarpJS Logo](https://cdn.warpjs.com/images/warpjs-logo-01.png)](https://warpjs.com)

_WarpJS_ introduces a unique approach to code and deploy back ends on serverless functions (FaaS), super easily.

> _WarpJS_ serverless is currently on free private beta. If you would like to test _WarpJS_, you need to know someone who is in the program to get an invitation link or to [request an invitation on WarpJS.com](https://warpjs.com/).

# Philosophy

In a nutshell, the developer writes the back-end logic in a Node.js module then the `warp` command line turns it into deployed serverless functions on the server side and generates a JavaScript API module wrapping the function calls on the client-side.

This way, developers can call scalable server-side functions in the most natural way from the client codebase, without coding any HTTP request or response.

The `warp` command line carries out three simple operations: `warp build`, `warp deploy` and `warp start-emulator`.

![Philosophy](https://cdn.warpjs.com/images/warpjs-principle-01.png)

# Quick start
### Clone an existing project
```bash
$ git clone https://github.com/WarpJS/warpjs-starter-template
$ cd warpjs-starter-template
$ npm install
```

### Login to WarpJS
_WarpJS_ comes with cloud hosting. Run the `warp login` command-line to install Warp and to authenticate to your [WarpJS account](https://starbase.warpjs.com/signin) at once.
```bash
$ npx warp login
```

### Run locally
```bash
# launches 'npx warp start-emulator' to emulate the FaaS locally and starts an HTTP server for the front
$ npm run dev
```

### Deploy on a cloud (GCP, AWS, Azure)
```bash
# build the serverless functions and a module exposing the API
$ npx warp build --output umd:dist/client.js .
# deploys the functions as FaaS, and the client assets optionally
$ npx warp deploy --asset-dir dist/ .
```

# How it works
Out of a module, `warp build` generates (1) the FaaS back-end code for each function and (2) an API script exposing the back-end function calls. This way, developers can call these functions right from a front end or a Node.js.

Then `warp deploy` will deploy (1) the FaaS back-end code for each function on a selected cloud and (2) the API access script on a CDN.

From this moment on, the back end is ready to use. The front end imports the API script via an import or a traditional `<script>` tag and directly calls the back-end functions which will run on the chosen FaaS.

During these workflows, there is no need to code HTTP requests or have knowledge of DevOps. JavaScript programming is enough to develop and deploy a cloud back end in minutes.

In order to set up the back end during the development phase, _WarpJS_ also provides the `warp start-emulator` command line to emulate a FaaS server with integrated live reload and rebuild. This allows developers to run tests and debug iterations faster, before deployment.

# Docs and Community
* [Website](http://warpjs.com/)
* [Documentation](https://warpjs.dev/)
* [GitHub organization](https://github.com/WarpJS)
* [Beta GitHub repository](https://github.com/warpjs/beta)
* [Code samples](https://github.com/WarpJS/samples)
* [Report an issue](https://github.com/WarpJS/beta/issues)
